diff --git a/dlls/winemac.drv/cocoa_window.m b/dlls/winemac.drv/cocoa_window.m
index ae69479..d721be3 100644
--- a/dlls/winemac.drv/cocoa_window.m
+++ b/dlls/winemac.drv/cocoa_window.m
@@ -18,6 +18,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#import <Cocoa/Cocoa.h>
 #import <Carbon/Carbon.h>
 
 #import "cocoa_window.h"
@@ -519,13 +520,15 @@ static inline void fix_generic_modifiers_by_device(NSUInteger* modifiers)
 
         if (self.disabled)
             style &= ~NSResizableWindowMask;
-        if (style != [self styleMask])
+        if (style != [self styleMask] && !([self styleMask] & NSFullScreenWindowMask))
             [self setStyleMask:style];
 
         if (style & NSClosableWindowMask)
             [[self standardWindowButton:NSWindowCloseButton] setEnabled:!self.disabled];
         if (style & NSMiniaturizableWindowMask)
             [[self standardWindowButton:NSWindowMiniaturizeButton] setEnabled:!self.disabled];
+        if (style & NSResizableWindowMask)
+            [[self standardWindowButton:NSWindowZoomButton] setEnabled:!self.disabled];
     }
 
     - (void) setWindowFeatures:(const struct macdrv_window_features*)wf
@@ -548,10 +551,12 @@ static inline void fix_generic_modifiers_by_device(NSUInteger* modifiers)
         fullscreen = (screen != nil);
         captured = (screen || [self screen]) && [controller areDisplaysCaptured];
 
-        if (captured || fullscreen)
+        if (captured || (fullscreen && !(normalStyleMask & NSResizableWindowMask)))
         {
             if (captured)
                 level = CGShieldingWindowLevel() + 1; /* Need +1 or we don't get mouse moves */
+            else if ([self styleMask] & NSFullScreenWindowMask)
+                level = NSMainMenuWindowLevel + 2;
             else
                 level = NSMainMenuWindowLevel + 1;
 
@@ -624,6 +629,8 @@ static inline void fix_generic_modifiers_by_device(NSUInteger* modifiers)
             behavior |= NSWindowCollectionBehaviorParticipatesInCycle;
             if ([self isVisible])
                 [NSApp addWindowsItem:self title:[self title] filename:NO];
+            if (normalStyleMask & NSResizableWindowMask)
+                behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
         }
         [self setCollectionBehavior:behavior];
 
@@ -958,7 +965,8 @@ static inline void fix_generic_modifiers_by_device(NSUInteger* modifiers)
         // full-screen mode.  In that case, we don't allow NSWindow to constrain
         // it.
         NSRect contentRect = [self contentRectForFrameRect:frameRect];
-        if (!screen_covered_by_rect(contentRect, [NSScreen screens]))
+        if ([self styleMask] & NSResizableWindowMask ||
+            !screen_covered_by_rect(contentRect, [NSScreen screens]))
             frameRect = [super constrainFrameRect:frameRect toScreen:screen];
         return frameRect;
     }
@@ -1234,6 +1242,11 @@ static inline void fix_generic_modifiers_by_device(NSUInteger* modifiers)
                                      forMode:NSRunLoopCommonModes];
     }
 
+    - (NSSize)window:(NSWindow *)window willUseFullScreenContentSize:(NSSize)proposedSize
+    {
+        return NSMakeSize(proposedSize.width, proposedSize.height);
+    }
+
 
     /*
      * ---------- NSPasteboardOwner methods ----------
